#ifndef QT_FEATURES_DataVisualization_src_datavisualization_qtdatavisualization_config_h_H
#define QT_FEATURES_DataVisualization_src_datavisualization_qtdatavisualization_config_h_H


#define QT_VERSION_STR "6.12.0"

#define QT_VERSION_MAJOR 6

#define QT_VERSION_MINOR 12

#define QT_VERSION_PATCH 0

#endif // QT_FEATURES_DataVisualization_src_datavisualization_qtdatavisualization_config_h_H
